<?php

namespace Model\Indicators;

use Nette\DateTime,
	ANN\Network,
	ANN\InputValue,
	ANN\OutputValue,
	ANN\Values;

/**
 * Neural network
 */
class NeuralNetwork
{
	private $predictionModel;
	private $predictionSettingsModel;
	private $predictionDataModel;
	private $hiddenLayerModel;

	public $trainStartTime;

	public function __construct(\Model\PredictionModel $predictionModel, \Model\PredictionSettingsModel $predictionSettingsModel,\Model\PredictionDataModel $predictionDataModel, \Model\HiddenLayerModel $hiddenLayerModel)
	{
		$this->predictionModel = $predictionModel;
		$this->predictionSettingsModel = $predictionSettingsModel;
		$this->predictionDataModel = $predictionDataModel;
		$this->hiddenLayerModel = $hiddenLayerModel;
	}

	function traincallback($ann, $train_data, $max_epochs, $epochs_between_reports, $desired_error, $epochs)
	{
		echo microtime(true)-$this->trainStartTime;exit;
		if (microtime(true)-$this->trainStartTime > 300){
			return FALSE;
		}

		return TRUE;
	}

	public function getValues($indicator, $prices)
	{
		$predictedValues = array();
		foreach ($prices['close'] as $key => $value) {
			if ($key >= $indicator->prediction_settings->historical_values-1){
				$arrayToPredict = array();
				for ($i = $key-$indicator->prediction_settings->historical_values+1; $i <= $key; $i++) {
					$arrayToPredict[] = $prices['close'][$i];
				}
				$predictedValues[$key] = $this->predict($indicator->prediction_settings, $arrayToPredict);
			}
		}
		return $predictedValues;
	}

	public function test($indicator, $indicatorValues, $priceKey, $price, $type)
	{
		switch ($type){
			case 'buy':
				if ($indicatorValues[$priceKey] > $price){
					//predikovana je vetsi nez soucasna
					return TRUE;
				} else {
					//predikovana je mensi nez soucasna
					return FALSE;
				}
				break;
			case 'sell':
				if ($indicatorValues[$priceKey] < $price){
					//predikovana je mensi nez soucasna
					return TRUE;
				} else {
					//predikovana je vetsi nez soucasna
					return FALSE;
				}
				break;
			default:
				return FALSE;
		}
	}

	public function predict($predictionSettingsRow, $dataArray)
	{
		if (sizeof($dataArray) !== $predictionSettingsRow->historical_values){
			return 'number_values_error';
		} else {
			//scale data
			$scaleValues = $this->predictionDataModel->scaleData($dataArray, $predictionSettingsRow->scale_min_value, $predictionSettingsRow->scale_max_value);
			if (!$scaleValues){
				$output = $this->fannRunNetwork($predictionSettingsRow, $dataArray);
			} else {
				$output = $this->fannRunNetwork($predictionSettingsRow, $scaleValues);
			}	

			if ($output == FALSE){
				return 'run_error';
			} else {
				return $scaleValues === FALSE ? $output[0] : $this->predictionDataModel->unscaleValue($predictionSettingsRow, $output[0]);
			}
		}
	}


	// ******************************************************************* //

	public function fannRunNetwork($predictionSettingsRow, $historicalValues)
	{
		$filePath = WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.net";
		if (!file_exists($filePath)){
			$this->fannTrainNetwork($predictionSettingsRow);
		}

		$ann = fann_create_from_file($filePath);
		if ($ann == FALSE){
			exit("Could not create ANN.");
		}

		return fann_run($ann, $historicalValues);
	}

	public function superTest()
	{
		$test_time_start = microtime(true);

		$historicalValuesArray = array(30);
		$desiredErrorArray = array(0.000025, 0.00005, 0.0001);
		$epochs = array(10000, 20000);
		$typeTrain = array('sequentially', 'random');
		$hiddenLayers = array(1, 2, 3); 
		$neuronsInHiddenLayer = array(1, 3, 5, 7, 9);

		$predictionRow = $this->predictionModel->add(array(
			'user_id' => 2,
			'name' => 'Super test EURUSD 15',
			'filename' => NULL, 
			'original_name' => 'EURUSD15.csv',
			'file_size' => 1128760,
			'file_checksum' => '74dfeb7881ce4a7533841db7874666a8',
			'column_number' => 6,
			'date_created' => new DateTime,
			));

		//presunu soubor
		$pathFrom = WWW_DIR . "/upload/super/EURUSD15.csv";
		$pathTo = WWW_DIR . "/upload/2/p/" . $predictionRow->id . "/EURUSD15.csv";
		//neni vyresena duplicita nazvu
		mkdir(WWW_DIR . "/upload/2/p/" . $predictionRow->id, 0755, true);
		copy($pathFrom, $pathTo);

		$predictionRow->update(array('filename' => $pathTo));

		$num = 0;

		foreach ($historicalValuesArray as $key => $historicalValues) {
			foreach ($desiredErrorArray as $key => $desiredError) {
				foreach ($epochs as $key => $epoch) {
					foreach ($typeTrain as $key => $type) {
						foreach ($hiddenLayers as $key => $hiddenLayer) {
							switch ($hiddenLayer){
								case 1:
									foreach ($neuronsInHiddenLayer as $key => $neurons) {
										//vytvorim test
										$predictionSettingsRow = $this->predictionSettingsModel->add(array(
											'prediction_id' => $predictionRow->id,
											'data_file' => 'EURUSD15.csv',
											'historical_values' => $historicalValues,
											'desired_error' => $desiredError,
											'learning_rate' => 0.7,
											'max_epochs' => $epoch,
											'activation_function_hidden' => 5,
											'activation_function_output' => 5,
											'percentage_train_data' => 80,
											'distribution_data' => $type,
											'state' => 'created'
											));
										$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neurons));
										$this->trainAndTestNN($predictionSettingsRow);
									}	
									break;
								case 2:
									foreach ($neuronsInHiddenLayer as $key => $neuronsLayer1) {
										foreach ($neuronsInHiddenLayer as $key => $neuronsLayer2) {
											$predictionSettingsRow = $this->predictionSettingsModel->add(array(
												'prediction_id' => $predictionRow->id,
												'data_file' => 'EURUSD15.csv',
												'historical_values' => $historicalValues,
												'desired_error' => $desiredError,
												'learning_rate' => 0.7,
												'max_epochs' => $epoch,
												'activation_function_hidden' => 5,
												'activation_function_output' => 5,
												'percentage_train_data' => 80,
												'distribution_data' => $type,
												'state' => 'created'
												));
											$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neuronsLayer1));
											$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neuronsLayer2));
											$this->trainAndTestNN($predictionSettingsRow);
										}
									}
									break;
								case 3:
									foreach ($neuronsInHiddenLayer as $key => $neuronsLayer1) {
										foreach ($neuronsInHiddenLayer as $key => $neuronsLayer2) {
											foreach ($neuronsInHiddenLayer as $key => $neuronsLayer3) {
												$predictionSettingsRow = $this->predictionSettingsModel->add(array(
													'prediction_id' => $predictionRow->id,
													'data_file' => 'EURUSD15.csv',
													'historical_values' => $historicalValues,
													'desired_error' => $desiredError,
													'learning_rate' => 0.7,
													'max_epochs' => $epoch,
													'activation_function_hidden' => 5,
													'activation_function_output' => 5,
													'percentage_train_data' => 80,
													'distribution_data' => $type,
													'state' => 'created'
													));
												$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neuronsLayer1));
												$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neuronsLayer2));
												$this->hiddenLayerModel->add(array('prediction_settings_id' => $predictionSettingsRow->id, 'neurons' => $neuronsLayer3));
												$this->trainAndTestNN($predictionSettingsRow);
											}
										}
									}
									break;
							}
							sleep(1);
							$num++;
							if ($num % 200 === 0){
								echo $num . PHP_EOL;
							}
						}
					}
				}
			}
		}
		$test_time_end = microtime(true);
		$test_time = $test_time_end - $test_time_start;
		echo "finito za " . $test_time . ' sekund' . PHP_EOL;
	}

	public function trainAndTestNN($predictionSettingsRow)
	{
		//natrenuju
		$ann = $this->train($predictionSettingsRow);
		//otestuju
		//$filePath = WWW_DIR . "/upload/" . $this->user->id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.net";
		
		//$ann = fann_create_from_file($filePath);

		if (!$ann){
			echo "chyba sit". PHP_EOL;
			exit;
		}

		$data = $this->predictionDataModel->csvToTestArray($predictionSettingsRow);

		if (!$data){
			echo "chyba data". PHP_EOL;
			exit;
		}

		$errorSum = 0;
		$maxError = 0;
		$minError = 999999999999999;
		$directionSuccess = 0;
		$directionFail = 0;

		foreach ($data as $key => $d) {
			$output = fann_run($ann, $d['input']);
			$result = $this->predictionDataModel->unscaleValue($predictionSettingsRow, $output[0]);
			$error = abs($result - $d['output']);

			$current = $this->predictionDataModel->unscaleValue($predictionSettingsRow, end($d['input']));

			if ((float)$d['output'] > $current){ //up
				if ($result > $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}
			} elseif ((float)$d['output'] < $current){ //down
				if ($result < $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}
			} else { //==
				/*if ($result == $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}*/
			}

			if ($error > $maxError){
				$maxError = $error;
			}	
			if ($error < $minError){
				$minError = $error;
			}
			$errorSum += $error;
		}

		if ($directionFail === 0){
			$directionSuccesRate = 1;
		} else {
			$directionSuccesRate = $directionSuccess / ($directionSuccess+$directionFail);
		}

		$averageError = $errorSum/count($data);

		$predictionSettingsRow->update(array(
			'test_average_error' => $averageError,
			'test_max_error' => $maxError,
			'test_min_error' => $minError,
			'test_direction_succes_rate' => $directionSuccesRate,
			'state' => 'done',
			));

		//unlink data file
		unlink(WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.data");
	}

	public function train($predictionSettingsRow)
	{
		$time_start = microtime(true);

		//settings
		$historical_values = $predictionSettingsRow->historical_values;
		$num_input = $historical_values;
		$num_output = 1;
		$desired_error = $predictionSettingsRow->desired_error;
		$max_epochs = $predictionSettingsRow->max_epochs;
		$epochs_between_reports = 50;

		$hidden_layers = $predictionSettingsRow->related('hidden_layer');

		$arrLayers[] = $num_input;
		foreach ($hidden_layers as $key => $row) {
			$arrLayers[] = $row->neurons;
		}
		$arrLayers[] = 1;

		$num_layers = $hidden_layers->count();
		
		//set NN
		$ann = fann_create_standard_array($num_layers+2, $arrLayers);

		//fann_set_callback($ann, $this->traincallback);
		fann_set_learning_rate($ann, $predictionSettingsRow->learning_rate);

		if ($ann) {
		    fann_set_activation_function_hidden($ann, $predictionSettingsRow->activation_function_hidden);
		    fann_set_activation_function_output($ann, $predictionSettingsRow->activation_function_output);

		    $fileFolder = WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id;
		    $filename = $fileFolder . "/ann.data";

		    if (!file_exists($filename)){
		    	$this->predictionDataModel->csvToFannFile($predictionSettingsRow);
		    }

		    $this->trainStartTime = microtime(true);
		    if (fann_train_on_file($ann, $filename, $max_epochs, $epochs_between_reports, $desired_error) != FALSE){
		    	$values['train_error'] = fann_get_MSE($ann);
		    	$filePath = $fileFolder . "/ann.net";
		    	fann_save($ann, $filePath);
		    } else {
		    	return FALSE;
		    }

		} else {
			return FALSE;
		}
		$time_end = microtime(true);
		$values['train_time'] = $time_end - $time_start;

		$predictionSettingsRow->update($values);

		return $ann;
	}

}
