<?php

namespace Model;


/**
 * Tabulka strategy_data
 */
class PredictionDataModel extends DataModel
{

	public function csvToFannFile($predictionSettingsRow)
	{
		$delimiter = ',';
		if(!file_exists($predictionSettingsRow->prediction->filename) || !is_readable($predictionSettingsRow->prediction->filename))
	        return FALSE;

	    $header = NULL;

	    $val = array();

    	if (($handle = fopen($predictionSettingsRow->prediction->filename, 'r')) !== FALSE){
	        while (($row = fgetcsv($handle, 1000, $delimiter)) !== FALSE)
	        {
	        	$val[] = $row[$predictionSettingsRow->prediction->column_number-1];
	    	}
	    	fclose($handle);
		}

		$scaleValues = $this->scaleData($val, min($val), max($val));
		if (!$scaleValues){
			$scaleValues = $val;
			$predictionSettingsRow->update(array(
				'scale' => FALSE,
				'scale_min_value' => min($val),
				'scale_max_value' => max($val),
				));
		} else {
			$predictionSettingsRow->update(array(
				'scale' => TRUE,
				'scale_min_value' => min($val),
				'scale_max_value' => max($val),
				));
		}
		
		$trainDataCount = round((count($scaleValues)-$predictionSettingsRow->historical_values)*($predictionSettingsRow->percentage_train_data/100));

		//kdyby nekdo zadal 100
		if ($trainDataCount == count($scaleValues)){
			$trainDataCount = $trainDataCount-1;
		}

		$fileFolder = WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id;

		if (!file_exists($fileFolder)) {
		    mkdir($fileFolder, 0777, true);
		}

		$annData = fopen($fileFolder . "/ann.data", "a"); 

		$countInputNeurons = $predictionSettingsRow->historical_values;
		$countOutputNeurons = 1;
		$text = $trainDataCount . " " . $countInputNeurons . " " . $countOutputNeurons . "\n";
		fwrite($annData, $text);

		$values = array();

		if ($predictionSettingsRow->distribution_data === 'sequentially'){
			for ($i = $predictionSettingsRow->historical_values-1; $i < $trainDataCount+$predictionSettingsRow->historical_values; $i++){
				$text = "";
				for ($j = $i-$predictionSettingsRow->historical_values+1; $j <= $i; $j++){
	    			if ($j === $i-$predictionSettingsRow->historical_values+1){
	    				$text = $text . $scaleValues[$j];
	    			} else {
	    				$text = $text . " " . $scaleValues[$j];
	    			}
	    		}

	    		$text = $text . "\n";
	    		fwrite($annData, $text);

	    		$outputString = $scaleValues[$i+1] . "\n";
	    		fwrite($annData, $outputString);
			}
		} elseif ($predictionSettingsRow->distribution_data === 'random'){
			for ($i = 0; $i < $trainDataCount; $i++){
				$startValue = mt_rand($predictionSettingsRow->historical_values-1,count($scaleValues)-2);
				$text = "";
				for ($j = $startValue-$predictionSettingsRow->historical_values+1; $j <= $startValue; $j++){
	    			if ($j === $startValue-$predictionSettingsRow->historical_values+1){
	    				$text = $text . $scaleValues[$j];
	    			} else {
	    				$text = $text . " " . $scaleValues[$j];
	    			}
	    		}

	    		$text = $text . "\n";
	    		fwrite($annData, $text);

	    		$outputString = $scaleValues[$startValue+1] . "\n";
	    		fwrite($annData, $outputString);
			}
		}

		fclose($annData);
		unset($scaleValues);
		unset($val);
	}

	public function csvToTestArray($predictionSettingsRow)
	{
		$delimiter = ',';
		if(!file_exists($predictionSettingsRow->prediction->filename) || !is_readable($predictionSettingsRow->prediction->filename))
	        return FALSE;

	    $header = NULL;

	    $val = array();

    	if (($handle = fopen($predictionSettingsRow->prediction->filename, 'r')) !== FALSE){
	        while (($row = fgetcsv($handle, 1000, $delimiter)) !== FALSE)
	        {
	        	$val[] = $row[$predictionSettingsRow->prediction->column_number-1];
	    	}
	    	fclose($handle);
		}

		$scaleValues = $this->scaleData($val, $predictionSettingsRow->scale_min_value, $predictionSettingsRow->scale_max_value); //scale data?
		if (!$scaleValues){
			$scaleValues = $val; //no scale, data are in range (0,1)
		} 
		
		$testDataCount = (count($scaleValues)-$predictionSettingsRow->historical_values) - round(count($scaleValues)*($predictionSettingsRow->percentage_train_data/100));

		if ($testDataCount == 0){
			return FALSE;
		}

		$values = array();

		if ($predictionSettingsRow->distribution_data === 'sequentially'){
			for ($i = count($scaleValues)-$testDataCount; $i < count($scaleValues)-1; $i++){
				$v = array();
				for ($j = $i-$predictionSettingsRow->historical_values+1; $j <= $i; $j++){
					$v['input'][] = $scaleValues[$j]; 
				}
				$v['output'] = $val[$i+1];

				$values[] = $v;
			}
		} elseif ($predictionSettingsRow->distribution_data === 'random') {
			for ($i = 0; $i < $testDataCount; $i++){
				$v = array();
				$startValue = mt_rand($predictionSettingsRow->historical_values-1,count($scaleValues)-2);
				for ($j = $startValue-$predictionSettingsRow->historical_values+1; $j <= $startValue; $j++){
					$v['input'][] = $scaleValues[$j]; 
				}
				$v['output'] = $val[$startValue+1];

				$values[] = $v;
			}
		}

		return $values;
	}

	public function scaleData($values, $min, $max)
	{
		//scaling into (0,1) range
		if ($max < 1 && $min > 0){
			return FALSE;
		}

		foreach ($values as $key => $value) {
			$values[$key] = ($value - $min)/($max-$min);
		}

		return $values;
	}

	//return value in normal form
	public function unscaleValue($predictionSettingsRow, $value)
	{
		if ($predictionSettingsRow->scale){
			return ($value * ($predictionSettingsRow->scale_max_value - $predictionSettingsRow->scale_min_value)) + $predictionSettingsRow->scale_min_value;
		} else {
			return $value;
		}
	}

	private function findScale($values)
	{
		//scaling into (-1,1) range
		$max = max($values);
		$min = min($values);

		if ($max < 1 && $min > -1){
			return 1;
		} elseif ($max < 10 && $min > -10) {
			return 10;
		} elseif ($max < 100 && $min > -100) {
			return 100;
		} elseif ($max < 1000 && $min > -1000) {
			return 1000;
		} elseif ($max < 10000 && $min > -10000) {
			return 10000;
		} elseif ($max < 100000 && $min > -100000) {
			return 100000;
		} elseif ($max < 1000000 && $min > -1000000) {
			return 1000000;
		} elseif ($max < 10000000 && $min > -10000000) {
			return 10000000;
		} elseif ($max < 100000000 && $min > -100000000) {
			return 100000000;
		} elseif ($max < 1000000000 && $min > -1000000000) {
			return 1000000000;
		} elseif ($max < 10000000000 && $min > -10000000000) {
			return 10000000000;
		}
	}
}