<?php

namespace FrontModule;

use \Nette\Application\UI\Form,
	\Utils\Nette\Message,
	\Nette\DateTime,
	\Nette\Forms\Container,
	ANN\Network,
	ANN\InputValue,
	ANN\OutputValue,
	ANN\Values,
	\Nette\Forms\Controls\SubmitButton;


class PredictionPresenter extends SecuredPresenter
{
	private $predictionModel;
	private $predictionDataModel;
	private $predictionSettingsModel;
	private $hiddenLayerModel;

	private $predictionRow;
	private $predictionSettingsRow;
	private $hiddenLayers;
	private $data;
	private $networkResults;

	private $trainStartTime;

	private $activationFunctions = array(
		0 => 'Linear',
		1 => 'Threshold',
		2 => 'Threshold symmetric',
		3 => 'Sigmoid',
		4 => 'Sigmoid stepwise',
		5 => 'Sigmoid symmetric',
		6 => 'Sigmoid symmetric stepwise',
		7 => 'Gaussian',
		8 => 'Gaussian symmetric',
		9 => 'Gaussian stepwise',
		10 => 'Elliot',
		11 => 'Elliot symmetric',
		12 => 'Linear piece',
		13 => 'Linear piece symmetric',
		14 => 'Sin symmetric',
		15 => 'Cos symmetric',
		16 => 'Sin',
		17 => 'Cos'
		);

	protected function startup()
	{
		parent::startup();
		$this->predictionModel = $this->context->predictionModel;
		$this->predictionDataModel = $this->context->predictionDataModel;
		$this->predictionSettingsModel = $this->context->predictionSettingsModel;
		$this->hiddenLayerModel =  $this->context->hiddenLayerModel;

		set_time_limit(300);
	}

	public function actionAll()
	{

	}

	public function renderAll()
	{
		$this->template->predictions = $this->predictionModel->findBy(array('user_id' => $this->user->id));
	}

	public function actionNew()
	{

	}

	public function renderNew()
	{

	}

	protected function createComponentNewPredictionForm()
	{
		$form = new Form;

		$form->addText('name', 'Prediction name', 15, 100)		
			->setAttribute('placeholder', 'Insert prediction name')		
			->setAttribute('class', 'form-control')		
			->setRequired('You must insert name of prediction');
		$form->addUpload('file', 'CSV File', FALSE);
		$form->addText('column_number', 'Number of column you want to predict')
			->setType('number') // <input type=number>
            ->setDefaultValue(1)
            ->addRule(Form::INTEGER, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(1, 100));

		$form->addSubmit('send', 'CONTINUE')
			->setAttribute('class', 'btn btn-lg btn-success');

		$form->onSuccess[] = callback($this, 'newPredictionFormSucceeded');
		return $form;
	}

	public function newPredictionFormSucceeded(Form $form)
	{
		$values = $form->getValues();

		//ulozeni predikce
		$prediction = array();
		$prediction['user_id'] = $this->user->id;
		$prediction['name'] = $values->name;	
		$prediction['filename'] = NULL;
		$prediction['original_name'] = $values->file->name;
		$prediction['file_size'] = $values->file->size;
		$prediction['file_checksum'] = md5_file($values->file);
		$prediction['column_number'] = $values->column_number;
		$prediction['date_created'] = new DateTime;

		$predictionRow = $this->predictionModel->add($prediction);

		//presunu soubor
		if ($values->file->isOk()){
			mkdir(WWW_DIR . "/upload/" . $this->user->id . "/p/" . $predictionRow->id, 0777, true);
			$path = WWW_DIR . "/upload/" . $this->user->id . "/p/" . $predictionRow->id . "/" . $values->file->name;
			//neni vyresena duplicita nazvu
			if ($path){
				$values->file->move($path);
			}
			$predictionRow->update(array('filename' => $path));
		} else {
			$path = NULL;
		}

		$this->redirect('Prediction:settings', $predictionRow->id);
	}

	public function actionSettings($id, $setting = NULL)
	{
		$this->predictionRow = $this->predictionModel->findById($id);
		$this->data = $this->predictionDataModel->csvToArray($this->predictionRow, $this->predictionRow->column_number-1, FALSE, 0, 1);

		if ($setting !== NULL){
			$this->predictionSettingsRow = $this->predictionSettingsModel->findById($setting);
			$this->hiddenLayers = $this->predictionSettingsRow->related('hidden_layer');
		} else {
			$this->predictionSettingsRow = NULL;
		}
	}

	public function renderSettings()
	{
		//$this->fannRunNetwork();
		$this->template->predictionRow = $this->predictionRow;
		$this->template->predictionSettingsRow = $this->predictionSettingsRow;
		$this->template->data = $this->data;
		$this->template->networkResults = $this->networkResults;
	}

	protected function createComponentPredictionSettingsForm()
	{
		$form = new Form;

		$form->addHidden('prediction_id');
		$form->addHidden('data_file');

		$form->addText('historical_values', 'Number of historical values to prediction:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(20)
            ->addRule(Form::INTEGER, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(1, 100));

        $removeEvent = callback($this, 'removeLayerClicked');

        $hiddenLayers = $form->addDynamic('hidden_layers', function (Container $container) use ($removeEvent) {
        	$container->addText('neurons', 'Number of neurons in hidden layer:')
	    		->setType('number') // <input type=number>
	            ->setDefaultValue(5)
	            ->addRule(Form::INTEGER, 'Must be numeric value')
	            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(0, 100));
	        /*$container->addSubmit('removeLayer', 'Remove')
            	->setValidationScope(FALSE)
            	->onClick[] = $removeEvent;*/
		},0);

		$hiddenLayers->addSubmit('addLayer', 'Add hidden layer')->setValidationScope(FALSE)->addCreateOnClick();

        /*$form->addText('hidden_layers', 'Number of hidden layers:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(5)
            ->addRule(Form::INTEGER, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(1, 100));

        $form->addText('hidden_layer_neurons', 'Number of neurons in hidden layers:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(5)
            ->addRule(Form::INTEGER, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(1, 100));*/

        $form->addText('desired_error', 'Size of desired error:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(0.0001)
            ->addRule(Form::FLOAT, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(0, 1));

        $form->addText('learning_rate', 'Neural network learning rate:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(0.7)
            ->setAttribute('step', 0.1)
            ->addRule(Form::FLOAT, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(0, 1));

        $form->addText('max_epochs', 'Max number of epochs (iterations) over training data during training:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(5000)
            ->addRule(Form::FLOAT, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(1, 30000));

        $form->addSelect('activation_function_hidden', 'Activation function for hidden layers:', $this->activationFunctions)
        	->setDefaultValue(5);

        $form->addSelect('activation_function_output', 'Activation function of output layer:', $this->activationFunctions)
        	->setDefaultValue(5);

        $form->addText('percentage_train_data', 'How much % of data do you want to use for training network?:')
    		->setType('number') // <input type=number>
            ->setDefaultValue(80)
            ->addRule(Form::INTEGER, 'Must be numeric value')
            ->addRule(Form::RANGE, 'Must be in range from %d to %d', array(0, 100));

        $form->addSelect('distribution_data', 'Distribution of training and test data:', array(
        	'sequentially' => 'Sequentially - training first x % of data, rest for testing',
        	'random' => 'Random - training and test data are randomly selected from dataset'));

        if ($this->predictionSettingsRow === NULL){
        	$form->setDefaults(array(
	        	'prediction_id' => $this->predictionRow->id,
	        	'data_file' => $this->predictionRow->original_name,
	        	'hidden_layers' => array(0 => array('neurons' => 5)),
	        	));
        } else {
        	foreach ($this->hiddenLayers as $key => $row) {
        		$hiddenLayersArr[$key]['neurons'] = $row->neurons;
        	}

        	$form->setDefaults(array(
	        	'prediction_id' => $this->predictionRow->id,
	        	'data_file' => $this->predictionRow->original_name,
	        	'historical_values' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->historical_values : NULL,
	        	'hidden_layers' => $hiddenLayersArr,
	        	'desired_error' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->desired_error : NULL,
	        	'learning_rate' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->learning_rate : NULL,
	        	'max_epochs' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->max_epochs : NULL,
	        	'activation_function_hidden' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->activation_function_hidden : NULL,
	        	'activation_function_output' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->activation_function_output : NULL,
	        	'percentage_train_data' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->percentage_train_data : NULL,
	        	'distribution_data' => $this->predictionSettingsRow !== NULL ? $this->predictionSettingsRow->distribution_data : NULL,
	        	));
        }
        

		$form->addSubmit('send', 'Train network')
			->setAttribute('class', 'btn btn-lg btn-success')
			->onClick[] = $this->predictionSettingsFormSucceeded;

		return $form;
	}

	public function addLayerClicked(SubmitButton $button)
	{
	    $button->parent->createOne();
	}

	/*public function removeLayerClicked(SubmitButton $button)
	{
	    // first parent is container
	    // second parent is it's replicator
	    $layers = $button->parent->parent;
	    $layers->remove($button->parent, TRUE);
	}*/

	public function predictionSettingsFormSucceeded(SubmitButton $button)
	{
		$values = $button->form->getValues();

	    $hiddenLayers = $values['hidden_layers'];
	    unset($values['hidden_layers']);
		$values['state'] = 'created';

		$ok = FALSE;
		foreach ($hiddenLayers as $key => $hidden_layer) {
			if ($hidden_layer->neurons > 0){
				$ok = TRUE;
				break;
			}
		}

		if (!$ok){
			$button->form['hidden_layers']['addLayer']->addError('You must have at least 1 hidden layer');
			return;
		}

		$predictionSettingsRow = $this->predictionSettingsModel->add($values);

		foreach ($hiddenLayers as $hidden_layer) { // values from replicator
			if ($hidden_layer->neurons > 0){
		    	$this->hiddenLayerModel->add(array(
		    		'prediction_settings_id' => $predictionSettingsRow->id,
		    		'neurons' => $hidden_layer->neurons,
		    		));
		    }
	    }

		$training = $this->fannTrainNetwork($predictionSettingsRow);

		if (!$training){
			$predictionSettingsRow->update(array('state' => 'failed'));
			$this->flashMessage('Neural network training error.');
		} else {
			$training['state'] = 'done';
			$predictionSettingsRow->update($training);
		}
		
		$this->redirect(':Front:Prediction:settings', $predictionSettingsRow->prediction_id, $predictionSettingsRow->id);
	}

	protected function createComponentPredictForm()
	{
		$form = new Form;

		$form->addHidden('prediction_settings_id');
		$form->addTextArea('historical_values', 'Strategy description',5)
			->setAttribute('class', 'form-control')
			->setAttribute('rows', 6)
    		->addRule(Form::MAX_LENGTH, 'Poznámka je příliš dlouhá', 10000);

    	$form->addSubmit('predict', 'Predict future value');

		if ($this->predictionSettingsRow !== NULL){
        	$form->setDefaults(array(
	        	'prediction_settings_id' => $this->predictionSettingsRow->id,
	        	));
        }

		$form->onSuccess[] = callback($this, 'predictFormSucceeded');
		return $form;
	}

	public function predictFormSucceeded(Form $form)
	{
		$values = $form->getValues();

		$predictionSettingsRow = $this->predictionSettingsModel->findById($values->prediction_settings_id);

		$dataArray = explode(";", $values->historical_values);
		foreach ($dataArray as $key => $value){
			$dataArray[$key] = trim($value);
		}

		if (sizeof($dataArray) !== $predictionSettingsRow->historical_values){
			$this->payload->predicted_value = 'number_values_error';
		} else {
			//scale data
			$scaleValues = $this->predictionDataModel->scaleData($dataArray, $predictionSettingsRow->scale_min_value, $predictionSettingsRow->scale_max_value);
			if (!$scaleValues){
				$output = $this->fannRunNetwork($predictionSettingsRow, $dataArray);
			} else {
				$output = $this->fannRunNetwork($predictionSettingsRow, $scaleValues);
			}	

			if ($output == FALSE){
				$this->payload->predicted_value = 'run_error';
			} else {
				$this->payload->predicted_value = $scaleValues === FALSE ? $output[0] : $this->predictionDataModel->unscaleValue($predictionSettingsRow, $output[0]);
			}
		}

		$this->sendPayload();
	}

	public function fannTrainNetwork($predictionSettingsRow)
	{
		$time_start = microtime(true);

		//settings
		$historical_values = $predictionSettingsRow->historical_values;
		$num_input = $historical_values;
		$num_output = 1;
		$desired_error = $predictionSettingsRow->desired_error;
		$max_epochs = $predictionSettingsRow->max_epochs;
		$epochs_between_reports = 50;

		$hidden_layers = $predictionSettingsRow->related('hidden_layer');

		$arrLayers[] = $num_input;
		foreach ($hidden_layers as $key => $row) {
			$arrLayers[] = $row->neurons;
		}
		$arrLayers[] = 1;

		$num_layers = $hidden_layers->count();
		
		//set NN
		$ann = fann_create_standard_array($num_layers+2, $arrLayers);

		fann_set_callback($ann, $this->traincallback);
		fann_set_learning_rate($ann, $predictionSettingsRow->learning_rate);

		if ($ann) {
		    fann_set_activation_function_hidden($ann, $predictionSettingsRow->activation_function_hidden);
		    fann_set_activation_function_output($ann, $predictionSettingsRow->activation_function_output);

		    $fileFolder = WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id; 

		    $filename = $fileFolder . "/ann.data";

		    if (!file_exists($filename)){
		    	$this->predictionDataModel->csvToFannFile($predictionSettingsRow);
		    }

		    $this->trainStartTime = microtime(true);
		    if (fann_train_on_file($ann, $filename, $max_epochs, $epochs_between_reports, $desired_error) != FALSE){
		    	$values['train_error'] = fann_get_MSE($ann);
		    	$filePath = $fileFolder . "/ann.net";
		    	fann_save($ann, $filePath);
		    } else {
		    	return FALSE;
		    }
		        
		    fann_destroy($ann);
		} else {
			return FALSE;
		}
		$time_end = microtime(true);
		$values['train_time'] = $time_end - $time_start;
		unlink(WWW_DIR . "/upload/" . $predictionSettingsRow->prediction->user_id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.data");
		
		return $values;
	}

	public function fannRunNetwork($predictionSettingsRow, $historicalValues)
	{
		$filePath = WWW_DIR . "/upload/" . $this->user->id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.net";
		if (!file_exists($filePath)){
			$this->fannTrainNetwork($predictionSettingsRow);
		}

		$ann = fann_create_from_file($filePath);
		if ($ann == FALSE){
			exit("Could not create ANN.");
		}

		return fann_run($ann, $historicalValues);
	}

	public function handleRunTest($predictionSettingsId)
	{
		$predictionSettingsRow = $this->predictionSettingsModel->findById($predictionSettingsId);

		$filePath = WWW_DIR . "/upload/" . $this->user->id . "/p/" . $predictionSettingsRow->prediction_id . "/" . $predictionSettingsRow->id . "/ann.net";
		if (!file_exists($filePath)){
			$this->fannTrainNetwork($predictionSettingsRow);
		}

		$ann = fann_create_from_file($filePath);
		if (!$ann){
			$this->flashMessage('Neural network training error.');
			$this->redirect(':Front:Prediction:settings', $predictionSettingsRow->prediction_id, $predictionSettingsRow->id);
		}

		$data = $this->predictionDataModel->csvToTestArray($predictionSettingsRow);

		if (!$data){
			$this->flashMessage('Neural network training error.');
			$this->redirect(':Front:Prediction:settings', $predictionSettingsRow->prediction_id, $predictionSettingsRow->id);
		}

		$errorSum = 0;
		$maxError = 0;
		$minError = 999999999999999;
		$directionSuccess = 0;
		$directionFail = 0;

		foreach ($data as $key => $d) {
			$output = fann_run($ann, $d['input']);
			$result = $this->predictionDataModel->unscaleValue($predictionSettingsRow, $output[0]);
			$error = abs($result - $d['output']);

			$current = $this->predictionDataModel->unscaleValue($predictionSettingsRow, end($d['input']));

			if ((float)$d['output'] > $current){ //up
				if ($result > $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}
			} elseif ((float)$d['output'] < $current){ //down
				if ($result < $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}
			} else { //==
				if ($result == $current){
					$directionSuccess++;
				} else {
					$directionFail++;
				}
			}

			if ($error > $maxError){
				$maxError = $error;
			}	
			if ($error < $minError){
				$minError = $error;
			}
			$errorSum += $error;
		}

		if ($directionFail === 0){
			$directionSuccesRate = 1;
		} else {
			$directionSuccesRate = $directionSuccess / ($directionSuccess+$directionFail);
		}

		$averageError = $errorSum/count($data);

		$predictionSettingsRow->update(array(
			'test_average_error' => $averageError,
			'test_max_error' => $maxError,
			'test_min_error' => $minError,
			'test_direction_succes_rate' => $directionSuccesRate,
			));

		$this->flashMessage('Neural network test completed.', 'alert-success');
		$this->redirect(':Front:Prediction:settings', $predictionSettingsRow->prediction_id, $predictionSettingsRow->id);
	}

	public function traincallback($ann, $train_data, $max_epochs, $epochs_between_reports, $desired_error, $epochs)
	{
		if (microtime(true)-$this->trainStartTime > 300){
			return FALSE;
		}

		return TRUE;
	}

	public function handleDeletePrediction($id)
	{
		$this->predictionModel->findById($id)->delete();
		$this->redirect('this');
	}
}
