<?php

namespace FrontModule;

use \Nette\Application\UI\Form,
	\Utils\Nette\Message,
	\Nette\DateTime,
	\Nette\Forms\Container;


class StrategyPresenter extends SecuredPresenter
{

	private $strategyModel;

	private $strategyDataModel;

	private $strategyIndicatorModel;

	private $strategyTestModel;

	private $strategyTestTransactionModel;

	private $predictionSettingsModel;

	//INDICATORS
	private $movingAverage;
	private $relativeStrengthIndex;
	private $tradersDynamicIndex;

	private $data;
	private $strategyRow;
	private $indicator;
	private $indicators;
	private $results;
	private $tests;
	private $strategyTestRow;
	private $transactions;

	protected function startup()
	{
		parent::startup();
		$this->strategyModel = $this->context->strategyModel;
		$this->strategyDataModel = $this->context->strategyDataModel;
		$this->strategyIndicatorModel = $this->context->strategyIndicatorModel;
		$this->strategyTestModel = $this->context->strategyTestModel;
		$this->strategyTestTransactionModel = $this->context->strategyTestTransactionModel;
		$this->predictionSettingsModel = $this->context->predictionSettingsModel;

		$this->movingAverage = $this->context->movingAverage;
		$this->relativeStrengthIndex = $this->context->relativeStrengthIndex;
		$this->tradersDynamicIndex = $this->context->tradersDynamicIndex;

		set_time_limit(0);
	}

	public function actionDefault()
	{
	}

	public function renderDefault()
	{
		$this->template->strategies = $this->strategyModel->findBy(array('user_id' => $this->user->id));
	}

	public function actionNew()
	{
	}

	public function renderNew()
	{
	}

	protected function createComponentNewStrategyForm()
	{
		$form = new Form;

		$form->addText('name', 'Strategy name', 15, 100)		
			->setAttribute('placeholder', 'Insert name')		
			->setAttribute('class', 'form-control')		
			->setRequired('You must insert name of strategy');
		$form->addTextArea('description', 'Strategy description',5)
			->setAttribute('class', 'form-control')
			->setAttribute('rows', 6)
    		->addRule(Form::MAX_LENGTH, 'Poznámka je příliš dlouhá', 10000);
		$form->addUpload('file', 'CSV File', FALSE);

		$form->addSubmit('send', 'CONTINUE')
			->setAttribute('class', 'btn btn-lg btn-success');

		$form->onSuccess[] = callback($this, 'newStrategyFormSucceeded');
		return $form;
	}

	public function newStrategyFormSucceeded(Form $form)
	{
		$values = $form->getValues();

		//ulozeni strategie
		$strategy = array();
		$strategy['name'] = $values->name;	
		$strategy['description'] = $values->description;	
		$strategy['user_id'] = $this->user->id;
		$strategy['filename'] = NULL;
		$strategy['original_name'] = $values->file->name;
		$strategy['file_size'] = $values->file->size;
		$strategy['file_checksum'] = md5_file($values->file);
		$strategy['date_created'] = new DateTime;

		$strategyRow = $this->strategyModel->add($strategy);

		//presunu soubor
		if ($values->file->isOk()){
			$path = WWW_DIR . "/upload/" . $this->user->id . "/s/" . $strategyRow->id . "/" . $values->file->name;
			//neni vyresena duplicita nazvu
			if (!file_exists($path)){
				$values->file->move($path);
			}
			$strategyRow->update(array('filename' => $path));
		} else {
			$path = NULL;
		}

		$this->redirect('Strategy:settings', $strategyRow->id);
	}

	public function actionSettings($id, $indicator = NULL)
	{
		if (is_null($id)){
			return $this->error();
		}

		$this->strategyRow = $this->strategyModel->findById($id);

		if ($this->strategyRow->user_id !== $this->user->id){
			$this->error();
		}

		$this->indicators = $this->strategyIndicatorModel->findBy(array('strategy_id' => $this->strategyRow->id));

		if ($indicator !== NULL){
			$this->indicator = $this->strategyIndicatorModel->findById($indicator);
			if ($this->indicator->strategy->id !== $this->strategyRow->id){
				$this->error();
			}
		}
	}

	public function renderSettings()
	{
		$this->template->data = $this->data;
		$this->template->strategyRow = $this->strategyRow;
		$this->template->indicators = $this->indicators;
	}

	protected function createComponentIndicatorForm()
	{
		$form = new Form;

		$form->addSelect('type', 'Indicator', array(
				'ma' => 'Moving Average', 
				'rsi' => 'RSI (Relative Strength Index)',
				//'tdi' => 'Traders Dynamic Index',
				'cci' => 'CCI (Commodity Channel Index)',
				'sar' => 'Parabolic SAR',
				'doji' => 'Candlestick pattern: Doji',
				'doji_star' => 'Candlestick pattern: Doji Star',
				'shooting_star' => 'Candlestick pattern: Shooting Star',
				'hammer' => 'Candlestick pattern: Hammer',
				'hanging_man' => 'Candlestick pattern: Hanging Man',
				'neural_network' => 'Neural network prediction',
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		//MOVING AVERAGE
		$form->addText('MA_period', 'Period')
				->setDefaultValue(14)
				->setAttribute('class', 'form-control only-number');
		$form->addSelect('MA_type', 'MA Method', array(
				'simple' => 'Simple',
				'exponential' => 'Exponential',
				//'smoothed' => 'Smoothed',
				'weighted' => 'Weighted'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		/*$form->addSelect('MA_apply_to', 'Apply to', array(
				'close' => 'Close',
				'open' => 'Open',
				'high' => 'High',
				'low' => 'Low'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');*/
		$form->addSelect('MA_rule', 'Rule', array(
				'price_equal_MA' => 'Price = MA',
				'price_over_MA' => 'Price > MA',
				'price_under_MA' => 'Price < MA'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		//RELATIVE STRENGTH INDEX
		$form->addText('RSI_period', 'Period')
				->setDefaultValue(14)
				->setAttribute('class', 'form-control only-number');
		/*$form->addSelect('RSI_apply_to', 'Apply to', array(
				'close' => 'Close',
				'open' => 'Open',
				'high' => 'High',
				'low' => 'Low'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');*/
		$form->addSelect('RSI_rule', 'Rule', array(
				'RSI_entering_70' => 'RSI entering 70+ area',
				'RSI_leaving_70' => 'RSI leaving 70+ area',
				'RSI_entering_30' => 'RSI entering 30- area',
				'RSI_leaving_30' => 'RSI leaving 30- area',
				'RSI_over' => 'RSI > ?',
				'RSI_under' => 'RSI < ?',
				'RSI_equal' => 'RSI = ?',
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		$form->addText('RSI_rule_value', 'Rule value')
				->setDefaultValue(0)
				->setAttribute('class', 'form-control only-number');
		//COMMODITY CHANNEL INDEX
		$form->addText('CCI_period', 'Period')
				->setDefaultValue(14)
				->setAttribute('class', 'form-control only-number');
		/*$form->addSelect('CCI_apply_to', 'Apply to', array(
				'close' => 'Close',
				'open' => 'Open',
				'high' => 'High',
				'low' => 'Low'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');*/
		$form->addSelect('CCI_rule', 'Rule', array(
				'CCI_entering_100' => 'CCI entering 100+ area',
				'CCI_leaving_100' => 'CCI leaving 100+ area',
				'CCI_entering_m100' => 'CCI entering 100- area',
				'CCI_leaving_m100' => 'CCI leaving 100- area',
				'CCI_over' => 'CCI > ?',
				'CCI_under' => 'CCI < ?',
				'CCI_equal' => 'CCI = ?',
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		$form->addText('CCI_rule_value', 'Rule value')
				->setDefaultValue(0)
				->setAttribute('class', 'form-control only-number');
		//PARABOLIC SAR
		$form->addSelect('SAR_rule', 'Rule', array(
				'SAR_up' => 'SAR up (bearish)',
				'SAR_down' => 'SAR down (bullish)',
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		$form->addText('SAR_acceleration', 'Acceleration (step)')
				->setType('number')
				->setAttribute('step', 0.01)
				->setDefaultValue(0.02)
				->setAttribute('class', 'form-control only-number');

		$form->addText('SAR_maximum', 'Maximum')
				->setType('number')
				->setAttribute('step', 0.1)
				->setDefaultValue(0.2)
				->setAttribute('class', 'form-control only-number');

		//DOJI

		//NEURAL NETWORK
		$predictionSettings = $this->predictionSettingsModel->findBy(array('prediction.file_checksum' => $this->strategyRow->file_checksum, 'state' => 'done'))->order('test_direction_succes_rate DESC');
		$predictionSettingsArray = array();
		foreach ($predictionSettings as $key => $row) {
			$predictionSettingsArray[$key] = 'TDSR: '. (round($row->test_direction_succes_rate*100, 3)) .'%, HV: ' . $row->historical_values . ', TE: ' . $row->train_error . ', TAE: ' . $row->test_average_error . ', TMAE: ' . $row->test_max_error . ', TMIE: ' . $row->test_min_error;
		}
		$form->addSelect('prediction_settings_id', 'Neural network settings', $predictionSettingsArray)
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');

		$form->addHidden('id');

		$form->addSubmit('send', 'SAVE INDICATOR')
			->setAttribute('class', 'btn btn-lg btn-success fright insert-button');

		if (!empty($this->indicator)) {
			$form['type']->setDisabled();
			switch ($this->indicator->type) {
				case 'ma':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
						'MA_period' => $this->indicator->period,
						'MA_type' => $this->indicator->type_indicator,
						//'MA_apply_to' => $this->indicator->apply_to,
						'MA_rule' => $this->indicator->rule,
					);
					break;
				
				case 'rsi':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
						'RSI_period' => $this->indicator->period,
						//'RSI_apply_to' => $this->indicator->apply_to,
						'RSI_rule' => $this->indicator->rule,
						'RSI_rule_value' => $this->indicator->rule_value,
					);
					break;
				case 'cci':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
						'CCI_period' => $this->indicator->period,
						//'CCI_apply_to' => $this->indicator->apply_to,
						'CCI_rule' => $this->indicator->rule,
						'CCI_rule_value' => $this->indicator->rule_value,
					);
					break;
				case 'sar':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
						'SAR_rule' => $this->indicator->rule,
						'SAR_acceleration' => $this->indicator->acceleration,
						'SAR_maximum' => $this->indicator->maximum,
					);
					break;
				case 'doji':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
				case 'doji_star':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
				case 'shooting_star':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
				case 'hammer':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
				case 'hanging_man':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
				case 'neural_network':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
						'prediction_settings_id' => $this->indicator->prediction_settings_id,
					);
					break;
				case 'tdi':
					$data = array(
						'id' => $this->indicator->id,
						'type' => $this->indicator->type,
					);
					break;
			}
			$form->setDefaults($data);
		}
		
		$form->onSuccess[] = callback($this, 'indicatorFormSucceeded');
		return $form;
	}

	public function indicatorFormSucceeded(Form $form)
	{
		$values = $form->getValues();

		foreach ($values as & $val) if ($val === '') $val = NULL;

		//validation
		if (empty($values->id) && empty($values->type)){
			$form['type']->addError('You must choose type of indicator.');
			return;
		}
		
		if (!isset($values->type)){
			$values->type = $this->indicator->type;
		}
	
		switch ($values->type){
			case 'ma':
				if (empty($values->MA_period)){
					$form->addError('Period of Moving average must be bigger than 0.');
				}
				if (empty($values->MA_type)){
					$form->addError('You must choose type of Moving Average.');
				}
				/*if (empty($values->MA_apply_to)){
					$form->addError('You must choose type of price.');
				}*/
				if (empty($values->MA_rule)){
					$form->addError('You must choose rule for Moving Average.');
				}
				break;
			case 'rsi':
				if (empty($values->RSI_period)){
					$form->addError('Period of RSI must be bigger than 0.');
				}
				/*if (empty($values->RSI_apply_to)){
					$form->addError('You must choose type of price.');
				}*/
				if (empty($values->RSI_rule)){
					$form->addError('You must choose rule for RSI.');
				}
				if (($values->RSI_rule === 'RSI_over' || $values->RSI_rule === 'RSI_under' || $values->RSI_rule === 'RSI_equal') && ($values->RSI_rule_value < 1) || ($values->RSI_rule_value > 100)){
					$form->addError('RSI must be in range from 1 to 100.');
				}
				break;
			case 'cci':
				if (empty($values->CCI_period)){
					$form->addError('Period of CCI must be bigger than 0.');
				}
				/*if (empty($values->CCI_apply_to)){
					$form->addError('You must choose type of price.');
				}*/
				if (empty($values->CCI_rule)){
					$form->addError('You must choose rule for CCI.');
				}
				if (($values->CCI_rule === 'CCI_over' || $values->CCI_rule === 'CCI_under' || $values->CCI_rule === 'CCI_equal') && ($values->CCI_rule_value < -500) || ($values->CCI_rule_value > 500)){
					$form->addError('CCI must be in range from -500 to 500.');
				}
				break;

			case 'sar':
				if (empty($values->SAR_rule)){
					$form->addError('You must choose rule for parabolic SAR.');
				}
				if (empty($values->SAR_acceleration) || $values->SAR_acceleration <= 0){
					$form->addError('Acceleration of parabolic SAR must be bigger than 0.');
				}
				if (empty($values->SAR_maximum) || $values->SAR_maximum <= 0){
					$form->addError('Maximum of parabolic SAR must be bigger than 0.');
				}
				break;
			case 'neural_network':
				if ($values->prediction_settings_id === NULL){
					$form->addError('You must choose setting of neural network.');
				}
				break;
		}

		if ($form->hasErrors()){
			return;
		}
		//end of validation
		if (empty($values->id)){
			//new
			$values->strategy_id = $this->strategyRow->id;
			$indicatorRow = $this->strategyIndicatorModel->insertIndicator($values);
		} else {
			//edit
			$indicatorRow = $this->strategyIndicatorModel->updateIndicator($values);
		}
		
		$this->flashMessage('Indicator saved.', 'alert-success');

		$this->redirect('Strategy:settings', array('id' => $this->strategyRow->id));
	}

	protected function createComponentStrategyForm()
	{
		$form = new Form;

		$form->addSelect('type', 'Order', array(
				'buy' => 'Buy', 
				'sell' => 'Sell'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		$form->addText('stop_loss', 'Stop Loss')
				->setType('number')
				->setDefaultValue(0)
				->setAttribute('class', 'form-control')
				->addRule(Form::FLOAT, 'Stop loss must be number');
		$form->addText('profit_target', 'Profit Target')
				->setType('number')
				->setDefaultValue(0)
				->setAttribute('class', 'form-control')
				->addRule(Form::FLOAT, 'Profit target must be number');
		$form->addSelect('strategy_apply_to', 'Calculate against price', array(
				'close' => 'Close',
				'open' => 'Open',
				'high' => 'High',
				'low' => 'Low'
				))
				->setPrompt('Choose...')
				->setAttribute('class', 'form-control');
		$form->addCheckbox('save_transactions', 'Save transactions of this strategy')
				->setDefaultValue(FALSE);

		$form->addSubmit('send_strategy', 'FIND RESULTS')
			->setAttribute('class', 'btn btn-lg btn-success fright insert-button');
		
		$form->onSuccess[] = callback($this, 'strategyFormSucceeded');
		return $form;
	}

	public function strategyFormSucceeded(Form $form)
	{
		$values = $form->getValues();
		//foreach ($values as & $val) if ($val === '') $val = NULL;

		//validation
		if (empty($values->type)){
			$form->addError('You must choose type of order.');
		}

		if ($values->stop_loss <= 0){
			$form->addError('Stop loss must be bigger than 0.');
		}

		if ($values->profit_target <= 0){
			$form->addError('Profit target must be bigger than 0.');
		}

		if (empty($values->strategy_apply_to)){
			$form->addError('You must choose type of price for calculating results.');
		}

		if ($form->hasErrors()){
			return;
		}
		//end of validation
		
		$testRow = $this->strategyTestModel->add(array(
			'strategy_id' => $this->strategyRow->id,
			'order_type' => $values->type,
			'stop_loss' => (float)$values->stop_loss,
			'profit_target' => (float)$values->profit_target,
			'total_trades' => NULL,
			'profit_percentage' => NULL,
			'profit_trades' => NULL,
			'loss_percentage' => NULL,
			'loss_trades' => NULL,
			'unrecognizable_percentage' => NULL,
			'unrecognizable_trades' => NULL,
			'open_percentage' => NULL,
			'open_trades' => NULL,
			'save_transactions' => $values->save_transactions,
			'created' => new DateTime(),
			'state' => 'created'
		));

		//PRICES TO DB
		/*
		if ($this->strategyDataModel->getTable()->fetch() === FALSE || $this->strategyDataModel->getTable()->fetch()->strategy_id !== $this->strategyRow->id){
			$this->strategyDataModel->csvToDB($this->strategyRow, $this->strategyRow->filename);
		}
		*/

		//PRICES TO ARRAY
		$prices = $this->strategyDataModel->csvToArrayTrader($this->strategyRow);

		//INDICATOR VALUES ARRAY
		$indicatorsValues = $this->strategyIndicatorModel->getIndicatorsValues($this->indicators, $prices);
	
		/*$pricesStrategy = $this->strategyDataModel->getPrices($values->strategy_apply_to, $prices);
		$pricesHigh = $this->strategyDataModel->getPrices('high', $prices);
		$pricesLow = $this->strategyDataModel->getPrices('low', $prices);*/
		$totalTrades = 0;
		$unrecognizableTrades = 0;
		$profitTrades = 0;
		$lossTrades = 0;
		$openTrades = 0;

		$numberOfIndicators = $this->indicators->count();

		foreach ($prices[$values->strategy_apply_to] as $priceKey => $price) {
			$indicatorSuccess = 0;
			foreach ($this->indicators as $indicatorKey => $indicatorRow) {
				if (isset($indicatorsValues[$indicatorKey][$priceKey])){
					if ($this->strategyIndicatorModel->testRule($indicatorRow, $indicatorsValues[$indicatorKey], $priceKey, $price, $values->type)){
						$indicatorSuccess++;
					} else {
						break; //invalid rule => no new order
					}
				} else {
					//if indicator value = NULL => stop testing this price
					break;
				}
			}

			if ($indicatorSuccess === $numberOfIndicators){
				//$precision = $this->strategyDataModel->getPrecision($prices);
				//$actualPrice = round($price, $precision);
				
				if ($values->type === 'buy'){
					//buy
					//$profitTarget = $actualPrice + ($values->profit_target / pow(10,$precision-1));
					//$stopLoss = $actualPrice - ($values->stop_loss / pow(10,$precision-1));
					$profitTarget = $price + $values->profit_target;
					$stopLoss = $price - $values->stop_loss;
					$date = $prices['date'][$priceKey];
					$time = $prices['time'][$priceKey];

					$openTrade = TRUE;
					for ($iBar = $priceKey+1; $iBar < count($prices[$values->strategy_apply_to]); $iBar++){
						if ($prices['high'][$iBar] > $profitTarget && $prices['low'][$iBar] < $stopLoss){
							$unrecognizableTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'buy', $date, $price, $profitTarget, $stopLoss, NULL, 'unrecognizable');
							}
							$openTrade = FALSE;
							break;
						} elseif ($prices['high'][$iBar] > $profitTarget){
							$profitTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'buy', $date, $price, $profitTarget, $stopLoss, 'profit', 'done');
							}
							$openTrade = FALSE;
							break;
						} elseif ($prices['low'][$iBar] < $stopLoss) {
							$lossTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'buy', $date, $price, $profitTarget, $stopLoss, 'loss', 'done');
							}
							$openTrade = FALSE;
							break;
						}
					}
					if ($openTrade === TRUE){
						$openTrades++;
						if ($values->save_transactions){
							$this->strategyTestTransactionModel->createTransaction($testRow->id, 'buy', $date, $price, $profitTarget, $stopLoss, NULL, 'open');
						}
						$openTrade = FALSE;
					}
				} else {
					//sell
					//$profitTarget = $actualPrice - ($values->profit_target / pow(10,$precision-1));
					//$stopLoss = $actualPrice + ($values->stop_loss / pow(10,$precision-1));
					$profitTarget = $price - $values->profit_target;
					$stopLoss = $price + $values->stop_loss;
					$date = $prices['date'][$priceKey];
					$time = $prices['time'][$priceKey];

					$openTrade = TRUE;
					for ($iBar = $priceKey+1; $iBar < count($prices[$values->strategy_apply_to]); $iBar++){
						if ($prices['low'][$iBar] < $profitTarget && $prices['high'][$iBar] > $stopLoss){
							$unrecognizableTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'sell', $date, $price, $profitTarget, $stopLoss, NULL, 'unrecognizable');
							}
							$openTrade = FALSE;
							break;
						} elseif ($prices['low'][$iBar] < $profitTarget){
							$profitTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'sell', $date, $price, $profitTarget, $stopLoss, 'profit', 'done');
							}
							$openTrade = FALSE;
							break;
						} elseif ($prices['high'][$iBar] > $stopLoss) {
							$lossTrades++;
							if ($values->save_transactions){
								$this->strategyTestTransactionModel->createTransaction($testRow->id, 'sell', $date, $price, $profitTarget, $stopLoss, 'loss', 'done');
							}
							$openTrade = FALSE;
							break;
						}
					}
					if ($openTrade === TRUE){
						$openTrades++;
						if ($values->save_transactions){
							$this->strategyTestTransactionModel->createTransaction($testRow->id, 'sell', $date, $price, $profitTarget, $stopLoss, NULL, 'open');
						}
						$openTrade = FALSE;
					}
				}
				$totalTrades++;
			}
		}
		$testRow->update(array(
			'total_trades' => $totalTrades,
			'profit_percentage' => $totalTrades !== 0 ? $profitTrades/$totalTrades : 0,
			'profit_trades' => $profitTrades,
			'loss_percentage' => $totalTrades !== 0 ? $lossTrades/$totalTrades : 0,
			'loss_trades' => $lossTrades,
			'unrecognizable_percentage' => $totalTrades !== 0 ? $unrecognizableTrades/$totalTrades : 0,
			'unrecognizable_trades' => $unrecognizableTrades,
			'open_percentage' => $totalTrades !== 0 ? $openTrades/$totalTrades : 0,
			'open_trades' => $openTrades,
			'created' => new DateTime(),
			'state' => 'done'
		));

		$this->redirect('Strategy:results', array('id' => $this->strategyRow->id, 'testId' => $testRow->id));
	}

	public function actionResults($id, $testId)
	{
		$this->strategyRow = $this->strategyModel->findById($id);
		$this->results = $this->strategyTestModel->findById($testId);
	}

	public function renderResults()
	{
		$this->template->strategyRow = $this->strategyRow;
		$this->template->results = $this->results;
	}

	public function actionTests($id = NULL)
	{
		if ($id === NULL){
			$this->tests = $this->strategyTestModel->getTable()->where('strategy.user_id = ?', $this->user->id);	
		} else {
			$this->tests = $this->strategyTestModel->findBy(array('strategy_id' => $id));
		}	
	}

	public function renderTests()
	{
		$this->template->tests = $this->tests;
	}
	
	public function actionTransactions($id = NULL)
	{
		if ($id === NULL){
			$this->error();
		} else {
			$this->strategyTestRow = $this->strategyTestModel->findById($id);
			$this->transactions = $this->strategyTestTransactionModel->findBy(array('strategy_test_id' => $id));
		}
	}

	public function renderTransactions()
	{
		$this->template->strategyTestRow = $this->strategyTestRow;
		$this->template->transactions = $this->transactions;
	}

	public function actionGraph($id)
	{
		$this->strategyRow = $this->strategyModel->findById($id);
		$this->data = $this->strategyDataModel->csvToArray($this->strategyRow, 5, FALSE, 0, 1);
	}

	public function renderGraph()
	{
		$this->template->data = $this->data;
		$this->template->strategyRow = $this->strategyRow;
	}

	public function handleDeleteStrategy($id)
	{
		$this->strategyModel->findById($id)->delete();

		$this->flashMessage('Strategy was succesfully deleted', 'alert-success');
		$this->redirect('this');
	}

	public function handleDeleteTest($id)
	{
		$this->strategyTestModel->findById($id)->delete();

		$this->flashMessage('Test was succesfully deleted', 'alert-success');
		$this->redirect('Strategy:tests');
	}

}
